@@ -12,19 +12,19 @@ def replace_column_names(x: DataFrame, y: Optional[Series]) -> Tuple[DataFrame,
     return x, y
 
 def normalize_col_name(text: str) -> str:
-    text = text.replace('\n', ' ').replace('\r', ' ').replace('\t', ' ')
-    text = text.replace('\u00A0', ' ')
+    for c in ['\n', '\r', '\t', '\u00A0']:
+        text = text.replace(c, ' ')
     text = replace_unspaced_symbols(text)
-    text = sanitize_text(text)
+    text = replace_whitespaces(text)
     return text
 
 def replace_unspaced_symbols(text: str) -> str:
     if ' ' not in text:
         return text
-    for c in ['_', '-', "."]:
+    for c in ['_', '-', ".", ":"]:
         text = text.replace(c, ' ')
     return text
 
-def sanitize_text(text: str) -> str:
-    # Remove control characters except for standard whitespace (newline, tab, etc.)
+def replace_whitespaces(text: str) -> str:
+    # TODO: originally this was supposed to keep newlines and tabs
     return re.sub(r'[\x00-\x1F\x7F]', ' ', text)
