@@ -1,4 +1,4 @@
-from typing import Optional, Tuple
+from typing import Optional, Tuple, Union
 
 import joblib
 import numpy as np
@@ -6,7 +6,6 @@ import torch
 from pandas import Series, DataFrame
 from peft import PeftModel
 from sklearn.base import BaseEstimator, ClassifierMixin, RegressorMixin
-from torch import autocast
 
 from tabstar.datasets.all_datasets import TabularDatasetID
 from tabstar.datasets.benchmark_folds import get_tabstar_version
@@ -15,28 +14,61 @@ from tabstar.preprocessing.splits import split_to_val
 from tabstar.tabstar_verbalizer import TabSTARVerbalizer, TabSTARData
 from tabstar.training.dataloader import get_dataloader
 from tabstar.training.devices import get_device
-from tabstar.training.metrics import apply_loss_fn, calculate_metric
+from tabstar.training.hyperparams import LORA_LR, LORA_R, MAX_EPOCHS, FINETUNE_PATIENCE, LORA_BATCH, GLOBAL_BATCH
+from tabstar.training.metrics import apply_loss_fn, calculate_metric, Metrics
 from tabstar.training.trainer import TabStarTrainer
-from tabstar.training.utils import concat_predictions
+from tabstar.training.utils import concat_predictions, fix_seed
 
 
 class BaseTabSTAR:
-    def __init__(self, verbose: bool = False,
-                 device: Optional[str] = None,
-                 pretrain_dataset: Optional[TabularDatasetID] = None):
+    def __init__(self,
+                 lora_lr: float = LORA_LR,
+                 lora_r: int = LORA_R,
+                 lora_batch: int = LORA_BATCH,
+                 global_batch: int = GLOBAL_BATCH,
+                 max_epochs: int = MAX_EPOCHS,
+                 patience: int = FINETUNE_PATIENCE,
+                 verbose: bool = False,
+                 device: Optional[Union[str,  torch.device]] = None,
+                 random_state: Optional[int] = None,
+                 pretrain_dataset_or_path: Optional[Union[str, TabularDatasetID]] = None,
+                 debug: bool = False):
+        self.lora_lr = lora_lr
+        self.lora_r = lora_r
+        self.lora_batch = lora_batch
+        self.global_batch = global_batch
+        self.max_epochs = max_epochs
+        self.patience = patience
         self.verbose = verbose
+        self.debug = debug
         self.preprocessor_: Optional[TabSTARVerbalizer] = None
         self.model_: Optional[PeftModel] = None
+        self.random_state = random_state
+        fix_seed(seed=self.random_state)
         self.device = get_device(device=device)
         print(f"🖥️ Using device: {self.device}")
-        self.model_version = get_tabstar_version(pretrain_dataset=pretrain_dataset)
+        self.use_amp = bool(self.device.type == "cuda")
+        self.model_version = get_tabstar_version(pretrain_dataset_or_path=pretrain_dataset_or_path)
 
     def fit(self, X, y):
-        train_data, val_data = self._prepare_for_train(X, y)
-        trainer = TabStarTrainer(device=self.device, model_version=self.model_version)
+        if self.model_ is not None:
+            raise ValueError("Model is already trained. Call fit() only once.")
+        self.vprint(f"Fitting model on data with shapes: X={X.shape}, y={y.shape}")
+        x = X.copy()
+        y = y.copy()
+        train_data, val_data = self._prepare_for_train(x, y)
+        self.vprint(f"We have: {len(train_data)} training and {len(val_data)} validation samples.")
+        trainer = TabStarTrainer(lora_lr=self.lora_lr,
+                                 lora_r=self.lora_r,
+                                 lora_batch=self.lora_batch,
+                                 global_batch=self.global_batch,
+                                 max_epochs=self.max_epochs,
+                                 patience=self.patience,
+                                 device=self.device,
+                                 model_version=self.model_version,
+                                 debug=self.debug)
         trainer.train(train_data, val_data)
-        trainer.load_model()
-        self.model_ = trainer.model
+        self.model_ = trainer.load_model()
 
     def predict(self, X):
         raise NotImplementedError("Must be implemented in subclass")
@@ -70,11 +102,12 @@ class BaseTabSTAR:
         return train_data, val_data
 
     def _infer(self, X) -> np.ndarray:
+        self.model_.eval()
         data = self.preprocessor_.transform(X, y=None)
         dataloader = get_dataloader(data, is_train=False, batch_size=128)
         predictions = []
         for data in dataloader:
-            with torch.no_grad(), autocast(device_type=self.device.type):
+            with torch.no_grad(), torch.autocast(device_type=self.device.type, enabled=self.use_amp):
                 batch_predictions = self.model_(x_txt=data.x_txt, x_num=data.x_num, d_output=data.d_output)
                 batch_predictions = apply_loss_fn(prediction=batch_predictions, d_output=data.d_output)
                 predictions.append(batch_predictions)
@@ -86,10 +119,15 @@ class BaseTabSTAR:
             print(s)
 
     def score(self, X, y) -> float:
-        y_pred = self.predict(X)
+        metrics = self.score_all_metrics(X=X, y=y)
+        return metrics.score
+
+    def score_all_metrics(self, X, y) -> Metrics:
+        x = X.copy()
+        y = y.copy()
+        y_pred = self._infer(x)
         y_true = self.preprocessor_.transform_target(y)
-        metric = calculate_metric(y_true, y_pred, d_output=self.preprocessor_.d_output)
-        return metric
+        return calculate_metric(y_true=y_true, y_pred=y_pred, d_output=self.preprocessor_.d_output)
 
 
 
@@ -110,6 +148,12 @@ class TabSTARClassifier(BaseTabSTAR, BaseEstimator, ClassifierMixin):
     def is_cls(self) -> bool:
         return True
 
+    @property
+    def classes_(self) -> np.ndarray:
+        if self.preprocessor_ is None or self.preprocessor_.y_values is None:
+            raise ValueError("Model is not trained yet! Call fit() before accessing classes_.")
+        return np.array(self.preprocessor_.y_values)
+
 
 class TabSTARRegressor(BaseTabSTAR, BaseEstimator, RegressorMixin):
 
